@@ -1,10 +1,14 @@
 #pragma once
 
-#include <juce_audio_processors/juce_audio_processors.h>
+#include <JuceHeader.h>
 
-#include "dsp/rosic_Open303.h"
+// Open303
+#include "dsp/open303/rosic_Open303.h"
 using namespace rosic;
 
+// GuitarML BYOD implementation
+#include "dsp/guitarml-byod/processors/drive/GuitarMLAmp.h"
+
 enum Open303Parameters
 {
   WAVEFORM = 0,
@@ -17,21 +21,24 @@ enum Open303Parameters
   VOLUME,
   // MODs
   SWITCH_MOD,
-  TANH_SHAPER_DRIVE,
-  AMP_SUSTAIN,
-  AMP_RELEASE,
-  SLIDE_TIME,
-  FEEDBACK_HPF,
-  SOFT_ATTACK,
   NORMAL_DECAY,
   ACCENT_DECAY,
-  //FILTER_TYPE,
+  FEEDBACK_HPF,
+  SOFT_ATTACK,
+  SLIDE_TIME,
+  TANH_SHAPER_DRIVE,
+  // Overdrive
+  OVERDRIVE_SWITCH,
+  OVERDRIVE_LEVEL,
+  OVERDRIVE_DRY_WET,
+  OVERDRIVE_MODEL_INDEX,
 
   OPEN303_NUM_PARAMETERS
 };
 
 //==============================================================================
-class JC303  :  public juce::AudioProcessor
+class JC303  :  public juce::AudioProcessor,
+                public juce::AudioProcessorValueTreeState::Listener
 {
 public:
     //==============================================================================
@@ -72,12 +79,30 @@ public:
     void getStateInformation (juce::MemoryBlock& destData) override;
     void setStateInformation (const void* data, int sizeInBytes) override;
 
+    //==============================================================================
+    void parameterChanged(const juce::String& parameterID, float newValue) override;
+
+    juce::StringArray getModelListNames() { return guitarML.getModelListNames(); }
+
 private:
-    void render(juce::AudioBuffer<float>& buffer, int beginSample, int endSample);
+    void render303(juce::AudioBuffer<float>& buffer, int beginSample, int endSample);
     void setParameter (Open303Parameters index, float value);
 
-    // the embedded core dsp object:
+    // presets and overdrive models user data management
+    void setupDataDirectories();
+    void installTones();
+    int loadOverdriveTones();
+
+    // embedded core dsp objects
+    // Open303
     Open303 open303Core;
+    // GuitarML - BYOD
+    GuitarMLAmp guitarML;
+    juce::dsp::DryWetMixer<float> overdriveMix;
+
+    // presets storage: user documents folder
+    File userAppDataDirectory = File::getSpecialLocation(File::userDocumentsDirectory).getChildFile(JucePlugin_Manufacturer).getChildFile(JucePlugin_Name);
+    File userAppDataDirectory_tones = userAppDataDirectory.getFullPathName() + "/overdrive_models";
 
     //==============================================================================
     juce::AudioProcessorValueTreeState parameters;
@@ -91,21 +116,24 @@ private:
     std::atomic<float>* volume = nullptr;
     // MODs
     std::atomic<float>* switchModState = nullptr;
-    std::atomic<float>* sqrDriver = nullptr;
-    std::atomic<float>* ampSustain = nullptr;
-    std::atomic<float>* ampRelease = nullptr;
-    std::atomic<float>* slideTime = nullptr;
-    std::atomic<float>* feedbackFilter = nullptr;
-    std::atomic<float>* softAttack = nullptr;
     std::atomic<float>* normalDecay = nullptr;
     std::atomic<float>* accentDecay = nullptr;
-    //std::atomic<float>* filterType = nullptr;
-    // mod fixed ranges support
-    bool lastSwitchModState = false;
+    std::atomic<float>* feedbackFilter = nullptr;
+    std::atomic<float>* softAttack = nullptr;
+    std::atomic<float>* slideTime = nullptr;
+    std::atomic<float>* sqrDriver = nullptr;
+    // overdrive
+    std::atomic<float>* overdriveModelIndex = nullptr;
+    std::atomic<float>* switchOverdriveState = nullptr;
+    std::atomic<float>* overdriveLevel = nullptr;
+    std::atomic<float>* overdriveDryWet = nullptr;
 
     double decayMin = 200;
     double decayMax = 2000;
 
+    // Flag to track if any parameter has changed
+    std::atomic<bool> parametersNeedUpdate { false };
+
     //==============================================================================
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (JC303)
 };
